@@ -1,20 +1,30 @@
+use crate::{ColorError, Convert, Primary, PrimaryColor, ReducedRGBA, Validate, RGBA};
 #[cfg(feature = "gdk")]
 use gdk;
 use serde::{Deserialize, Serialize};
-use crate::{ColorError, Convert, Primary, ReducedRGBA, RGBA, Validate};
 use std::u8;
 
 /// This struct contains a color represented in hex notation plus an opacity
 /// value. This is necessary to represent colors in an SVG image
 #[derive(Clone, Deserialize, Debug, PartialEq, Serialize)]
 pub struct HexColor {
+    /// A seven character string beginning with '#' representing the three
+    /// colors red, green and blue in hexadecimal
     pub color: String,
+    /// The opacity of the color
     pub alpha: f32,
 }
 
+impl ToString for HexColor {
+    fn to_string(&self) -> String {
+        format!("color: {}, alpha: {:.3}", self.color, self.alpha)
+    }
+}
+
 fn parse_hex(hex: &str) -> Result<(u8, u8, u8), ColorError> {
     validate_hex_string(hex)?;
-    Ok((match u8::from_str_radix(&hex[1..3], 16) {
+    Ok((
+        match u8::from_str_radix(&hex[1..3], 16) {
             Ok(c) => c,
             Err(_) => return Err(ColorError::InvalidHexCharacter),
         },
@@ -33,7 +43,7 @@ fn validate_hex_string(hex: &str) -> Result<(), ColorError> {
     match &hex.len() {
         x if *x < 7 => return Err(ColorError::TruncatedHexString),
         x if *x > 7 => return Err(ColorError::HexStringOverflow),
-        _ => {},
+        _ => {}
     };
     if &hex[0..1] != "#" {
         return Err(ColorError::InvalidHexCharacter);
@@ -54,7 +64,7 @@ impl Validate for HexColor {
         } else if self.alpha > 1.0 {
             Err(ColorError::OutsideBoundsHigh)
         } else {
-             match parse_hex(&self.color) {
+            match parse_hex(&self.color) {
                 Err(e) => Err(e),
                 Ok(_) => Ok(()),
             }
@@ -104,58 +114,18 @@ impl Convert for HexColor {
 }
 
 impl Primary for HexColor {
-    fn black() -> Self {
+    fn primary(color: PrimaryColor) -> Self {
         Self {
-            color: String::from("#000000"),
-            alpha: 1.0,
-        }
-    }
-
-    fn white() -> Self {
-        Self {
-            color: String::from("#ffffff"),
-            alpha: 1.0,
-        }
-    }
-
-    fn red() -> Self {
-        Self {
-            color: String::from("#ff0000"),
-            alpha: 1.0,
-        }
-    }
-
-    fn green() -> Self {
-        Self {
-            color: String::from("#00ff00"),
-            alpha: 1.0,
-        }
-    }
-
-    fn blue() -> Self {
-        Self {
-            color: String::from("#0000ff"),
-            alpha: 1.0,
-        }
-    }
-
-    fn yellow() -> Self {
-        Self {
-            color: String::from("#ffff00"),
-            alpha: 1.0,
-        }
-    }
-
-    fn magenta() -> Self {
-        Self {
-            color: String::from("#ff00ff"),
-            alpha: 1.0,
-        }
-    }
-
-    fn cyan() -> Self {
-        Self {
-            color: String::from("#00ffff"),
+            color: String::from(match color {
+                PrimaryColor::Black => "#000000",
+                PrimaryColor::White => "#ffffff",
+                PrimaryColor::Red => "#ff0000",
+                PrimaryColor::Green => "#00ff00",
+                PrimaryColor::Blue => "#0000ff",
+                PrimaryColor::Yellow => "#ffff00",
+                PrimaryColor::Magenta => "#ff00ff",
+                PrimaryColor::Cyan => "#00ffff",
+            }),
             alpha: 1.0,
         }
     }
@@ -167,81 +137,111 @@ mod tests {
 
     #[test]
     fn black() {
-        let k = HexColor::black();
-        assert_eq!(k, HexColor {
-            color: String::from("#000000"),
-            alpha: 1.0,
-        });
+        let k = HexColor::primary(PrimaryColor::Black);
+        assert_eq!(
+            k,
+            HexColor {
+                color: String::from("#000000"),
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn white() {
-        let w = HexColor::white();
-        assert_eq!(w, HexColor {
-            color: String::from("#ffffff"),
-            alpha: 1.0,
-        });
+        let w = HexColor::primary(PrimaryColor::White);
+        assert_eq!(
+            w,
+            HexColor {
+                color: String::from("#ffffff"),
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn red() {
-        let red = HexColor::red();
-        assert_eq!(red, HexColor {
-            color: String::from("#ff0000"),
-            alpha: 1.0,
-        });
+        let red = HexColor::primary(PrimaryColor::Red);
+        assert_eq!(
+            red,
+            HexColor {
+                color: String::from("#ff0000"),
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn green() {
-        let grn = HexColor::green();
-            assert_eq!(grn, HexColor {
-            color: String::from("#00ff00"),
-            alpha: 1.0,
-        });
+        let grn = HexColor::primary(PrimaryColor::Green);
+        assert_eq!(
+            grn,
+            HexColor {
+                color: String::from("#00ff00"),
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn blue() {
-        let blue = HexColor::blue();
-            assert_eq!(blue, HexColor {
-            color: String::from("#0000ff"),
-            alpha: 1.0,
-        });
+        let blue = HexColor::primary(PrimaryColor::Blue);
+        assert_eq!(
+            blue,
+            HexColor {
+                color: String::from("#0000ff"),
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn yellow() {
-        let yel = HexColor::yellow();
-            assert_eq!(yel, HexColor {
-            color: String::from("#ffff00"),
-            alpha: 1.0,
-        });
+        let yel = HexColor::primary(PrimaryColor::Yellow);
+        assert_eq!(
+            yel,
+            HexColor {
+                color: String::from("#ffff00"),
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn magenta() {
-        let mag = HexColor::magenta();
-            assert_eq!(mag, HexColor {
-            color: String::from("#ff00ff"),
-            alpha: 1.0,
-        });
+        let mag = HexColor::primary(PrimaryColor::Magenta);
+        assert_eq!(
+            mag,
+            HexColor {
+                color: String::from("#ff00ff"),
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn cyan() {
-        let c = HexColor::cyan();
-            assert_eq!(c, HexColor {
-            color: String::from("#00ffff"),
-            alpha: 1.0,
-        });
+        let c = HexColor::primary(PrimaryColor::Cyan);
+        assert_eq!(
+            c,
+            HexColor {
+                color: String::from("#00ffff"),
+                alpha: 1.0,
+            }
+        );
+    }
+
+    #[test]
+    fn to_string() {
+        let red = HexColor::red().to_string();
+        assert_eq!(red, "color: #ff0000, alpha: 1.000");
     }
 
     #[test]
     fn to_hex() {
-        let foo = HexColor::blue();
+        let foo = HexColor::primary(PrimaryColor::Blue);
         let bar = foo.to_hex();
-        assert_eq!(bar, Ok(HexColor::blue()));
+        assert_eq!(bar, Ok(HexColor::primary(PrimaryColor::Blue)));
     }
 
     #[test]
@@ -276,9 +276,9 @@ mod tests {
 
     #[test]
     fn to_rgba() {
-        let foo = HexColor::green();
+        let foo = HexColor::primary(PrimaryColor::Green);
         let bar = foo.to_rgba();
-        assert_eq!(bar, Ok(RGBA::green()));
+        assert_eq!(bar, Ok(RGBA::primary(PrimaryColor::Green)));
     }
 
     #[test]
@@ -313,9 +313,9 @@ mod tests {
 
     #[test]
     fn to_reduced() {
-        let red_hex = HexColor::red();
+        let red_hex = HexColor::primary(PrimaryColor::Red);
         let red = red_hex.to_reduced_rgba();
-        assert_eq!(red, Ok(ReducedRGBA::red()));
+        assert_eq!(red, Ok(ReducedRGBA::primary(PrimaryColor::Red)));
     }
 
     #[test]
@@ -351,13 +351,11 @@ mod tests {
     #[cfg(feature = "gdk")]
     #[test]
     fn to_gdk() {
-        let red = HexColor::red().to_gdk();
-        assert_eq!(red, Ok(gdk::RGBA {
-            red: 1.0,
-            green: 0.0,
-            blue: 0.0,
-            alpha: 1.0,
-        }));
+        let red = HexColor::primary(PrimaryColor::Red).to_gdk().unwrap();
+        assert_eq!(red.red(), 1.0);
+        assert_eq!(red.green(), 0.0);
+        assert_eq!(red.blue(), 0.0);
+        assert_eq!(red.alpha(), 1.0);
     }
 
     #[cfg(feature = "gdk")]
