@@ -1,7 +1,7 @@
+use crate::{ColorError, Convert, HexColor, Primary, PrimaryColor, ReducedRGBA, Validate};
 #[cfg(feature = "gdk")]
 use gdk;
 use serde::{Deserialize, Serialize};
-use crate::{ColorError, Convert, HexColor, Primary, ReducedRGBA, Validate};
 
 /// This struct represents colors in floating point precision as separate
 /// Red, Green, and Blue channels plus a separate Alpha (Opacity) channel
@@ -13,6 +13,12 @@ pub struct RGBA {
     pub alpha: f32,
 }
 
+impl ToString for RGBA {
+    fn to_string(&self) -> String {
+        format!("RGBA({:.3}, {:.3}, {:.3}, {:.3})", self.red, self.green, self.blue, self.alpha)
+    }
+}
+
 impl Validate for RGBA {
     type Err = ColorError;
 
@@ -91,90 +97,39 @@ impl Convert for RGBA {
     #[cfg(feature = "gdk")]
     fn to_gdk(&self) -> Result<gdk::RGBA, Self::Err> {
         self.validate()?;
-        Ok(gdk::RGBA {
-            red: self.red,
-            green: self.green,
-            blue: self.blue,
-            alpha: self.alpha,
-        })
+        Ok(gdk::builders::RGBABuilder::new()
+            .red(self.red)
+            .green(self.green)
+            .blue(self.blue)
+            .alpha(self.alpha)
+            .build())
     }
 }
 
 impl Primary for RGBA {
-    fn black() -> Self {
+    fn primary(color: PrimaryColor) -> Self {
         Self {
-            red: 0.0,
-            green: 0.0,
-            blue: 0.0,
-            alpha: 1.0,
-        }
-    }
-
-    fn white() -> Self {
-        Self {
-            red: 1.0,
-            green: 1.0,
-            blue: 1.0,
-            alpha: 1.0,
-        }
-    }
-
-    /// Returns "red" as an [RGBA] struct
-    fn red() -> Self {
-        Self {
-            red: 1.0,
-            green: 0.0,
-            blue: 0.0,
-            alpha: 1.0,
-        }
-    }
-
-    /// Returns "green" as an [RGBA] struct
-    fn green() -> Self {
-        Self {
-            red: 0.0,
-            green: 1.0,
-            blue: 0.0,
-            alpha: 1.0,
-        }
-    }
-
-    /// Returns "blue" as an [RGBA] struct
-    fn blue() -> Self {
-        Self {
-            red: 0.0,
-            green: 0.0,
-            blue: 1.0,
-            alpha: 1.0,
-        }
-    }
-
-    /// Returns "yellow" as an [RGBA] struct
-    fn yellow() -> Self {
-        Self {
-            red: 1.0,
-            green: 1.0,
-            blue: 0.0,
-            alpha: 1.0,
-        }
-    }
-
-    /// Returns "magenta" as an [RGBA] struct
-    fn magenta() -> Self {
-        Self {
-            red: 1.0,
-            green: 0.0,
-            blue: 1.0,
-            alpha: 1.0,
-        }
-    }
-
-    /// Returns "cyan" as an [RGBA] struct
-    fn cyan() -> Self {
-        Self {
-            red: 0.0,
-            green: 1.0,
-            blue: 1.0,
+            red: match color {
+                PrimaryColor::Black
+                | PrimaryColor::Green
+                | PrimaryColor::Blue
+                | PrimaryColor::Cyan => 0.0,
+                _ => 1.0,
+            },
+            green: match color {
+                PrimaryColor::Black
+                | PrimaryColor::Red
+                | PrimaryColor::Blue
+                | PrimaryColor::Magenta => 0.0,
+                _ => 1.0,
+            },
+            blue: match color {
+                PrimaryColor::Black
+                | PrimaryColor::Red
+                | PrimaryColor::Green
+                | PrimaryColor::Yellow => 0.0,
+                _ => 1.0,
+            },
             alpha: 1.0,
         }
     }
@@ -186,96 +141,126 @@ mod tests {
 
     #[test]
     fn black() {
-        let k = RGBA::black();
-        assert_eq!(k, RGBA {
-            red: 0.0,
-            green: 0.0,
-            blue: 0.0,
-            alpha: 1.0,
-        });
+        let k = RGBA::primary(PrimaryColor::Black);
+        assert_eq!(
+            k,
+            RGBA {
+                red: 0.0,
+                green: 0.0,
+                blue: 0.0,
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn white() {
-        let w = RGBA::white();
-        assert_eq!(w, RGBA {
-            red: 1.0,
-            green: 1.0,
-            blue: 1.0,
-            alpha: 1.0,
-        });
+        let w = RGBA::primary(PrimaryColor::White);
+        assert_eq!(
+            w,
+            RGBA {
+                red: 1.0,
+                green: 1.0,
+                blue: 1.0,
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn red() {
-        let red = RGBA::red();
-        assert_eq!(red, RGBA {
-            red: 1.0,
-            green: 0.0,
-            blue: 0.0,
-            alpha: 1.0,
-        });
+        let red = RGBA::primary(PrimaryColor::Red);
+        assert_eq!(
+            red,
+            RGBA {
+                red: 1.0,
+                green: 0.0,
+                blue: 0.0,
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn green() {
-        let grn = RGBA::green();
-            assert_eq!(grn, RGBA {
-            red: 0.0,
-            green: 1.0,
-            blue: 0.0,
-            alpha: 1.0,
-        });
+        let grn = RGBA::primary(PrimaryColor::Green);
+        assert_eq!(
+            grn,
+            RGBA {
+                red: 0.0,
+                green: 1.0,
+                blue: 0.0,
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn blue() {
-        let blue = RGBA::blue();
-            assert_eq!(blue, RGBA {
-            red: 0.0,
-            green: 0.0,
-            blue: 1.0,
-            alpha: 1.0,
-        });
+        let blue = RGBA::primary(PrimaryColor::Blue);
+        assert_eq!(
+            blue,
+            RGBA {
+                red: 0.0,
+                green: 0.0,
+                blue: 1.0,
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn yellow() {
-        let yel = RGBA::yellow();
-            assert_eq!(yel, RGBA {
-            red: 1.0,
-            green: 1.0,
-            blue: 0.0,
-            alpha: 1.0,
-        });
+        let yel = RGBA::primary(PrimaryColor::Yellow);
+        assert_eq!(
+            yel,
+            RGBA {
+                red: 1.0,
+                green: 1.0,
+                blue: 0.0,
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn magenta() {
-        let mag = RGBA::magenta();
-            assert_eq!(mag, RGBA {
-            red: 1.0,
-            green: 0.0,
-            blue: 1.0,
-            alpha: 1.0,
-        });
+        let mag = RGBA::primary(PrimaryColor::Magenta);
+        assert_eq!(
+            mag,
+            RGBA {
+                red: 1.0,
+                green: 0.0,
+                blue: 1.0,
+                alpha: 1.0,
+            }
+        );
     }
 
     #[test]
     fn cyan() {
-        let c = RGBA::cyan();
-            assert_eq!(c, RGBA {
-            red: 0.0,
-            green: 1.0,
-            blue: 1.0,
-            alpha: 1.0,
-        });
+        let c = RGBA::primary(PrimaryColor::Cyan);
+        assert_eq!(
+            c,
+            RGBA {
+                red: 0.0,
+                green: 1.0,
+                blue: 1.0,
+                alpha: 1.0,
+            }
+        );
+    }
+
+    #[test]
+    fn to_string() {
+        let blk = RGBA::black().to_string();
+        assert_eq!(blk, "RGBA(0.000, 0.000, 0.000, 1.000)");
     }
 
     #[test]
     fn to_hex() {
-        let red = RGBA::red().to_hex();
-        assert_eq!(red, Ok(HexColor::red()));
+        let red = RGBA::primary(PrimaryColor::Red).to_hex();
+        assert_eq!(red, Ok(HexColor::primary(PrimaryColor::Red)));
     }
 
     #[test]
@@ -302,7 +287,7 @@ mod tests {
 
     #[test]
     fn to_rgba() {
-        let red = RGBA::red();
+        let red = RGBA::primary(PrimaryColor::Red);
         let rgba = red.to_rgba().unwrap();
         assert_eq!(red, rgba);
     }
@@ -331,8 +316,8 @@ mod tests {
 
     #[test]
     fn to_reduced_rgba() {
-        let red = RGBA::red().to_reduced_rgba();
-        assert_eq!(red, Ok(ReducedRGBA::red()));
+        let red = RGBA::primary(PrimaryColor::Red).to_reduced_rgba();
+        assert_eq!(red, Ok(ReducedRGBA::primary(PrimaryColor::Red)));
     }
 
     #[test]
@@ -343,7 +328,10 @@ mod tests {
             blue: -1.0,
             alpha: 1.0,
         };
-        assert_eq!(invalid.to_reduced_rgba(), Err(ColorError::OutsideBoundsNegative));
+        assert_eq!(
+            invalid.to_reduced_rgba(),
+            Err(ColorError::OutsideBoundsNegative)
+        );
     }
 
     #[test]
@@ -354,18 +342,21 @@ mod tests {
             blue: 1.0,
             alpha: 1.0,
         };
-        assert_eq!(invalid.to_reduced_rgba(), Err(ColorError::OutsideBoundsHigh));
+        assert_eq!(
+            invalid.to_reduced_rgba(),
+            Err(ColorError::OutsideBoundsHigh)
+        );
     }
 
     #[cfg(feature = "gdk")]
     #[test]
     fn rgba_to_gdk() {
-        let red = RGBA::red();
+        let red = RGBA::primary(PrimaryColor::Red);
         let gdk_red = red.to_gdk().unwrap();
-        assert_eq!(red.red, gdk_red.red);
-        assert_eq!(red.green, gdk_red.green);
-        assert_eq!(red.blue, gdk_red.blue);
-        assert_eq!(red.alpha, gdk_red.alpha);
+        assert_eq!(red.red, gdk_red.red());
+        assert_eq!(red.green, gdk_red.green());
+        assert_eq!(red.blue, gdk_red.blue());
+        assert_eq!(red.alpha, gdk_red.alpha());
     }
 
     #[cfg(feature = "gdk")]
